"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const type_1 = require("../core/type");
// cannot use import because compare and format are @isomorphic
//const moment = require('moment');
const TimeZone = new type_1.default({
    name: 'timezone',
    typescript: 'string',
    // @isomorphic
    compare(field, a, b) {
        return a.localeCompare(b, undefined, { sensitivity: 'base' });
    },
    query(path, where, query) {
        if (where) {
            query[path.name] = _.isArray(where) ? { $in: where } : where;
        }
    },
    matches(path, where, doc) {
        if (where) {
            let value = path.get(doc);
            if (value) {
                if (!_.isString(value)) {
                    value = '' + value;
                }
                if (_.isArray(where)) {
                    return where.indexOf(value) >= 0;
                }
                else if (where instanceof RegExp) {
                    return value.match(where);
                }
                else {
                    return value === where;
                }
            }
        }
        return true;
    },
});
exports.default = TimeZone;
//# sourceMappingURL=timezone.js.map