"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fetch = require("node-fetch");
const moment = require("moment");
const tyr_1 = require("../tyr");
const collection_1 = require("../core/collection");
let rates = {
    USD: {
        EUR: 0.84,
    },
};
const ExchangeRate = new collection_1.default({
    id: '_u5',
    name: 'tyrExchangeRate',
    client: false,
    internal: true,
    fields: {
        _id: { is: 'string' },
        updatedOn: { is: 'datetime' },
        rates: { is: 'object' },
    },
});
async function updateExchangeRates() {
    try {
        let exchangeRates = await ExchangeRate.byId('STANDARD');
        const cutoff = moment().subtract(tyr_1.default.options.fixer.every || 8 * 60 * 60 /* default 8 hours */, 's');
        if (!exchangeRates || cutoff.isAfter(exchangeRates.updatedOn)) {
            const result = await fetch(`http://data.fixer.io/api/latest?access_key=${tyr_1.default.options.fixer.accessKey}`);
            const json = await result.json();
            // we cache the results from the api in the db so that only one api call needs to be made
            // per tyranid cluster instead of per tyranid instance
            exchangeRates = await ExchangeRate.save({
                _id: 'STANDARD',
                updatedOn: new Date(),
                rates: {
                    [json.base]: json.rates,
                },
            });
        }
        rates = exchangeRates.rates;
    }
    catch (err) {
        console.error('exchange rate error', err);
    }
}
async function boot() {
    const fixer = tyr_1.default.options.fixer;
    if (fixer) {
        if (!fixer.accessKey) {
            console.error('Fixer configuration provided but no accessKey given');
            return;
        }
        await updateExchangeRates();
        setTimeout(() => {
            setInterval(updateExchangeRates, 60 * 60 * 100 // every hour
            );
        }, 
        // stagger the queries randomly by up to 30s so that different servers don't simultaneously query fixer.io
        Math.floor(Math.random() * 30000));
    }
}
exports.boot = boot;
function getExchangeRate(from, to) {
    const fromRates = rates[from];
    if (fromRates) {
        const rate = fromRates[to];
        if (rate !== undefined) {
            return rate;
        }
    }
    const toRates = rates[to];
    if (toRates) {
        const rate = toRates[from];
        if (rate !== undefined) {
            return 1 / rate;
        }
    }
    //return undefined;
}
exports.getExchangeRate = getExchangeRate;
exports.default = {
    boot,
    getExchangeRate,
};
//# sourceMappingURL=exchangeRate.js.map